@@ -1,3 +1,4 @@
+{-# LANGUAGE LambdaCase #-}
 -- incomplete patterns in 'fromDistinct{Asc,Desc}List'
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 
@@ -251,10 +252,11 @@ map'
   -> Map k a
   -- ^ Object of the case analysis
   -> b
-map' e _ _ _ E = e
-map' e f g h (L l k a r) = f l k a r (map' e f g h l) (map' e f g h r)
-map' e f g h (B l k a r) = g l k a r (map' e f g h l) (map' e f g h r)
-map' e f g h (R l k a r) = h l k a r (map' e f g h l) (map' e f g h r)
+map' e f g h = \case
+  L l k a r -> f l k a r (map' e f g h l) (map' e f g h r)
+  R l k a r -> h l k a r (map' e f g h l) (map' e f g h r)
+  B l k a r -> g l k a r (map' e f g h l) (map' e f g h r)
+  E -> e
 
 {-
  - Construction
@@ -465,8 +467,8 @@ adjustWithKey f k0 = map' E (go L) (go B) (go R)
  where
   go c l k a r recl recr = case compare k0 k of
     LT -> c recl k a r
-    EQ -> c l k (f k a) r
     GT -> c l k a recr
+    EQ -> c l k (f k a) r
 
 -- | /O(log n)/ Adjust with an operation the value of the maximum key in a map
 adjustMax :: (a -> a) -> Map k a -> Map k a
@@ -595,8 +597,8 @@ split k0 =
     . foldrWithKey
       ( \k a (lt, a', gt) -> case compare k k0 of
           LT -> ((k, a) : lt, a', gt)
-          EQ -> (lt, Just a, gt)
           GT -> (lt, a', (k, a) : gt)
+          EQ -> (lt, Just a, gt)
       )
       ([], Nothing, [])
 
@@ -638,8 +640,8 @@ lookup k = map' Nothing go go go
  where
   go _ k' a _ recl recr = case compare k k' of
     LT -> recl
-    EQ -> Just a
     GT -> recr
+    EQ -> Just a
 
 -- | /O(log n)/ Fetch the least bin greater than or equal to a key
 lookupGE :: (Ord k) => k -> Map k a -> Maybe (k, a)
@@ -647,8 +649,8 @@ lookupGE k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr
-    EQ -> Just (k, a)
     GT -> recl <|> Just (k, a)
+    EQ -> Just (k, a)
 
 -- | /O(log n)/ Fetch the least bin strictly greater than a key
 lookupGT :: (Ord k) => k -> Map k a -> Maybe (k, a)
@@ -656,8 +658,8 @@ lookupGT k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr
-    EQ -> recr
     GT -> recl <|> Just (k, a)
+    EQ -> recr
 
 -- | /O(log n)/ Fetch the greatest bin less than or equal to a key
 lookupLE :: (Ord k) => k -> Map k a -> Maybe (k, a)
@@ -665,8 +667,8 @@ lookupLE k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr <|> Just (k, a)
-    EQ -> Just (k, a)
     GT -> recl
+    EQ -> Just (k, a)
 
 -- | /O(log n)/ Fetch the greatest bin strictly less than a key
 lookupLT :: (Ord k) => k -> Map k a -> Maybe (k, a)
@@ -674,8 +676,8 @@ lookupLT k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr <|> Just (k, a)
-    EQ -> recl
     GT -> recl
+    EQ -> recl
 
 -- | /O(log n)/ Fetch the bin with the maximum key, or 'Nothing' if empty
 lookupMax :: Map k a -> Maybe (k, a)
@@ -699,8 +701,8 @@ member k0 = map' False go go go
  where
   go _ k _ _ recl recr = case compare k0 k of
     LT -> recl
-    EQ -> True
     GT -> recr
+    EQ -> True
 
 -- | /O(1)/ Check whether a map is empty
 null :: Map k a -> Bool
@@ -795,20 +797,20 @@ delete' k0 =
     ( \l k a r _ _ ->
         case compare k0 k of
           LT -> deleteLl l k a r
-          EQ -> substituteL l r
           GT -> deleteLr l k a r
+          EQ -> substituteL l r
     )
     ( \l k a r _ _ ->
         case compare k0 k of
           LT -> deleteBl l k a r
-          EQ -> substituteBr l r
           GT -> deleteBr l k a r
+          EQ -> substituteBr l r
     )
     ( \l k a r _ _ ->
         case compare k0 k of
           LT -> deleteRl l k a r
-          EQ -> substituteR l r
           GT -> deleteRr l k a r
+          EQ -> substituteR l r
     )
  where
   deleteRl l k a r =
@@ -817,20 +819,20 @@ delete' k0 =
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftR (deleteLl ll lk la lr) k a r
-            EQ -> checkLeftR (substituteL ll lr) k a r
             GT -> checkLeftR (deleteLr ll lk la lr) k a r
+            EQ -> checkLeftR (substituteL ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> R (deleteBl ll lk la lr) k a r
-            EQ -> checkLeftR' (substituteBr ll lr) k a r
             GT -> R (deleteBr ll lk la lr) k a r
+            EQ -> checkLeftR' (substituteBr ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftR (deleteRl ll lk la lr) k a r
-            EQ -> checkLeftR (substituteR ll lr) k a r
             GT -> checkLeftR (deleteRr ll lk la lr) k a r
+            EQ -> checkLeftR (substituteR ll lr) k a r
       )
       l
   deleteRr l k a =
@@ -839,20 +841,20 @@ delete' k0 =
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightR l k a (deleteLl rl rk ra rr)
-            EQ -> checkRightR l k a (substituteL rl rr)
             GT -> checkRightR l k a (deleteLr rl rk ra rr)
+            EQ -> checkRightR l k a (substituteL rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> R l k a (deleteBl rl rk ra rr)
-            EQ -> checkRightR' l k a (substituteBl rl rr)
             GT -> R l k a (deleteBr rl rk ra rr)
+            EQ -> checkRightR' l k a (substituteBl rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightR l k a (deleteRl rl rk ra rr)
-            EQ -> checkRightR l k a (substituteR rl rr)
             GT -> checkRightR l k a (deleteRr rl rk ra rr)
+            EQ -> checkRightR l k a (substituteR rl rr)
       )
   deleteBl l k a r =
     map'
@@ -860,20 +862,20 @@ delete' k0 =
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftB (deleteLl ll lk la lr) k a r
-            EQ -> checkLeftB (substituteL ll lr) k a r
             GT -> checkLeftB (deleteLr ll lk la lr) k a r
+            EQ -> checkLeftB (substituteL ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> B (deleteBl ll lk la lr) k a r
-            EQ -> checkLeftB' (substituteBr ll lr) k a r
             GT -> B (deleteBr ll lk la lr) k a r
+            EQ -> checkLeftB' (substituteBr ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftB (deleteRl ll lk la lr) k a r
-            EQ -> checkLeftB (substituteR ll lr) k a r
             GT -> checkLeftB (deleteRr ll lk la lr) k a r
+            EQ -> checkLeftB (substituteR ll lr) k a r
       )
       l
   deleteBr l k a =
@@ -882,20 +884,20 @@ delete' k0 =
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightB l k a (deleteLl rl rk ra rr)
-            EQ -> checkRightB l k a (substituteL rl rr)
             GT -> checkRightB l k a (deleteLr rl rk ra rr)
+            EQ -> checkRightB l k a (substituteL rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> B l k a (deleteBl rl rk ra rr)
-            EQ -> checkRightB' l k a (substituteBl rl rr)
             GT -> B l k a (deleteBr rl rk ra rr)
+            EQ -> checkRightB' l k a (substituteBl rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightB l k a (deleteRl rl rk ra rr)
-            EQ -> checkRightB l k a (substituteR rl rr)
             GT -> checkRightB l k a (deleteRr rl rk ra rr)
+            EQ -> checkRightB l k a (substituteR rl rr)
       )
   deleteLl l k a r =
     map'
@@ -903,20 +905,20 @@ delete' k0 =
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftL (deleteLl ll lk la lr) k a r
-            EQ -> checkLeftL (substituteL ll lr) k a r
             GT -> checkLeftL (deleteLr ll lk la lr) k a r
+            EQ -> checkLeftL (substituteL ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> L (deleteBl ll lk la lr) k a r
-            EQ -> checkLeftL' (substituteBr ll lr) k a r
             GT -> L (deleteBr ll lk la lr) k a r
+            EQ -> checkLeftL' (substituteBr ll lr) k a r
       )
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> checkLeftL (deleteRl ll lk la lr) k a r
-            EQ -> checkLeftL (substituteR ll lr) k a r
             GT -> checkLeftL (deleteRr ll lk la lr) k a r
+            EQ -> checkLeftL (substituteR ll lr) k a r
       )
       l
   deleteLr l k a =
@@ -925,20 +927,20 @@ delete' k0 =
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightL l k a (deleteLl rl rk ra rr)
-            EQ -> checkRightL l k a (substituteL rl rr)
             GT -> checkRightL l k a (deleteLr rl rk ra rr)
+            EQ -> checkRightL l k a (substituteL rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> L l k a (deleteBl rl rk ra rr)
-            EQ -> checkRightL' l k a (substituteBl rl rr)
             GT -> L l k a (deleteBr rl rk ra rr)
+            EQ -> checkRightL' l k a (substituteBl rl rr)
       )
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> checkRightL l k a (deleteRl rl rk ra rr)
-            EQ -> checkRightL l k a (substituteR rl rr)
             GT -> checkRightL l k a (deleteRr rl rk ra rr)
+            EQ -> checkRightL l k a (substituteR rl rr)
       )
   rebalanceR l k a =
     map'
@@ -1294,18 +1296,18 @@ insertWithKey f k0 a0 =
   insertR l k a r =
     case compare k0 k of
       LT -> insertRl l k a r
-      EQ -> R l k (f k a0 a) r
       GT -> insertRr l k a r
+      EQ -> R l k (f k a0 a) r
   insertB l k a r =
     case compare k0 k of
       LT -> insertBl l k a r
-      EQ -> B l k (f k a0 a) r
       GT -> insertBr l k a r
+      EQ -> B l k (f k a0 a) r
   insertL l k a r =
     case compare k0 k of
       LT -> insertLl l k a r
-      EQ -> L l k (f k a0 a) r
       GT -> insertLr l k a r
+      EQ -> L l k (f k a0 a) r
   insertRl l k a r =
     map'
       (B (B E k0 a0 E) k a r)
@@ -1371,8 +1373,8 @@ insertWithKey f k0 a0 =
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
             LT -> insertRrl l k a rl rk ra rr
-            EQ -> R l k a (B rl rk (f rk a0 ra) rr)
             GT -> insertRrr l k a rl rk ra rr
+            EQ -> R l k a (B rl rk (f rk a0 ra) rr)
       )
       (\rl rk ra rr _ _ -> R l k a (insertR rl rk ra rr))
   insertLl l k a r =
@@ -1382,8 +1384,8 @@ insertWithKey f k0 a0 =
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
             LT -> insertLll ll lk la lr k a r
-            EQ -> L (B ll lk (f lk a0 la) lr) k a r
             GT -> insertLlr ll lk la lr k a r
+            EQ -> L (B ll lk (f lk a0 la) lr) k a r
       )
       (\ll lk la lr _ _ -> L (insertR ll lk la lr) k a r)
       l
