@@ -1,3 +1,5 @@
+{-# LANGUAGE LambdaCase #-}
+
 -- | Primitive recursive functions on various data structures
 module Mini.Data.Recursion (
   -- * Re-exports
@@ -183,8 +185,9 @@ list
   -> [a]
   -- ^ Object of the case analysis
   -> b
-list e _ [] = e
-list e f (a : as) = f a as (list e f as)
+list e f = \case
+  a : as -> f a as (list e f as)
+  [] -> e
 
 -- | Primitive recursion on non-empty lists
 nonEmpty
@@ -210,9 +213,10 @@ ordering
   -> Ordering
   -- ^ Object of the case analysis
   -> a
-ordering lt _ _ LT = lt
-ordering _ eq _ EQ = eq
-ordering _ _ gt GT = gt
+ordering lt eq gt = \case
+  LT -> lt
+  GT -> gt
+  EQ -> eq
 
 {-
  - n-Tuples (3 to 64)
