@@ -1,3 +1,4 @@
+{-# LANGUAGE LambdaCase #-}
 -- incomplete patterns in 'from{Asc,Desc}List'
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 
@@ -196,10 +197,11 @@ set'
   -> Set a
   -- ^ Object of the case analysis
   -> b
-set' e _ _ _ E = e
-set' e f g h (L l a r) = f l a r (set' e f g h l) (set' e f g h r)
-set' e f g h (B l a r) = g l a r (set' e f g h l) (set' e f g h r)
-set' e f g h (R l a r) = h l a r (set' e f g h l) (set' e f g h r)
+set' e f g h = \case
+  L l a r -> f l a r (set' e f g h l) (set' e f g h r)
+  R l a r -> h l a r (set' e f g h l) (set' e f g h r)
+  B l a r -> g l a r (set' e f g h l) (set' e f g h r)
+  E -> e
 
 {-
  - Construction
@@ -337,8 +339,8 @@ split a0 =
     . foldr
       ( \a (lt, a', gt) -> case compare a a0 of
           LT -> (a : lt, a', gt)
-          EQ -> (lt, True, gt)
           GT -> (lt, a', a : gt)
+          EQ -> (lt, True, gt)
       )
       ([], False, [])
 
@@ -372,8 +374,8 @@ lookupGE a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr
-    EQ -> Just a
     GT -> recl <|> Just a
+    EQ -> Just a
 
 -- | /O(log n)/ Fetch the least element strictly greater than the given one
 lookupGT :: (Ord a) => a -> Set a -> Maybe a
@@ -381,8 +383,8 @@ lookupGT a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr
-    EQ -> recr
     GT -> recl <|> Just a
+    EQ -> recr
 
 -- | /O(log n)/ Fetch the greatest element less than or equal to the given one
 lookupLE :: (Ord a) => a -> Set a -> Maybe a
@@ -390,8 +392,8 @@ lookupLE a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr <|> Just a
-    EQ -> Just a
     GT -> recl
+    EQ -> Just a
 
 -- | /O(log n)/ Fetch the greatest element strictly less than the given one
 lookupLT :: (Ord a) => a -> Set a -> Maybe a
@@ -399,8 +401,8 @@ lookupLT a0 = set' Nothing go go go
  where
   go _ a _ recl recr = case compare a a0 of
     LT -> recr <|> Just a
-    EQ -> recl
     GT -> recl
+    EQ -> recl
 
 -- | /O(log n)/ Fetch the maximum element, or 'Nothing' if the set is empty
 lookupMax :: Set a -> Maybe a
@@ -424,8 +426,8 @@ member a0 = set' False go go go
  where
   go _ a _ recl recr = case compare a0 a of
     LT -> recl
-    EQ -> True
     GT -> recr
+    EQ -> True
 
 -- | /O(1)/ Check whether a set is empty
 null :: Set a -> Bool
@@ -492,20 +494,20 @@ delete' a0 =
     ( \l a r _ _ ->
         case compare a0 a of
           LT -> deleteLl l a r
-          EQ -> substituteL l r
           GT -> deleteLr l a r
+          EQ -> substituteL l r
     )
     ( \l a r _ _ ->
         case compare a0 a of
           LT -> deleteBl l a r
-          EQ -> substituteBr l r
           GT -> deleteBr l a r
+          EQ -> substituteBr l r
     )
     ( \l a r _ _ ->
         case compare a0 a of
           LT -> deleteRl l a r
-          EQ -> substituteR l r
           GT -> deleteRr l a r
+          EQ -> substituteR l r
     )
  where
   deleteRl l a r =
@@ -514,20 +516,20 @@ delete' a0 =
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftR (deleteLl ll la lr) a r
-            EQ -> checkLeftR (substituteL ll lr) a r
             GT -> checkLeftR (deleteLr ll la lr) a r
+            EQ -> checkLeftR (substituteL ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> R (deleteBl ll la lr) a r
-            EQ -> checkLeftR' (substituteBr ll lr) a r
             GT -> R (deleteBr ll la lr) a r
+            EQ -> checkLeftR' (substituteBr ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftR (deleteRl ll la lr) a r
-            EQ -> checkLeftR (substituteR ll lr) a r
             GT -> checkLeftR (deleteRr ll la lr) a r
+            EQ -> checkLeftR (substituteR ll lr) a r
       )
       l
   deleteRr l a =
@@ -536,20 +538,20 @@ delete' a0 =
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightR l a (deleteLl rl ra rr)
-            EQ -> checkRightR l a (substituteL rl rr)
             GT -> checkRightR l a (deleteLr rl ra rr)
+            EQ -> checkRightR l a (substituteL rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> R l a (deleteBl rl ra rr)
-            EQ -> checkRightR' l a (substituteBl rl rr)
             GT -> R l a (deleteBr rl ra rr)
+            EQ -> checkRightR' l a (substituteBl rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightR l a (deleteRl rl ra rr)
-            EQ -> checkRightR l a (substituteR rl rr)
             GT -> checkRightR l a (deleteRr rl ra rr)
+            EQ -> checkRightR l a (substituteR rl rr)
       )
   deleteBl l a r =
     set'
@@ -557,20 +559,20 @@ delete' a0 =
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftB (deleteLl ll la lr) a r
-            EQ -> checkLeftB (substituteL ll lr) a r
             GT -> checkLeftB (deleteLr ll la lr) a r
+            EQ -> checkLeftB (substituteL ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> B (deleteBl ll la lr) a r
-            EQ -> checkLeftB' (substituteBr ll lr) a r
             GT -> B (deleteBr ll la lr) a r
+            EQ -> checkLeftB' (substituteBr ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftB (deleteRl ll la lr) a r
-            EQ -> checkLeftB (substituteR ll lr) a r
             GT -> checkLeftB (deleteRr ll la lr) a r
+            EQ -> checkLeftB (substituteR ll lr) a r
       )
       l
   deleteBr l a =
@@ -579,20 +581,20 @@ delete' a0 =
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightB l a (deleteLl rl ra rr)
-            EQ -> checkRightB l a (substituteL rl rr)
             GT -> checkRightB l a (deleteLr rl ra rr)
+            EQ -> checkRightB l a (substituteL rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> B l a (deleteBl rl ra rr)
-            EQ -> checkRightB' l a (substituteBl rl rr)
             GT -> B l a (deleteBr rl ra rr)
+            EQ -> checkRightB' l a (substituteBl rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightB l a (deleteRl rl ra rr)
-            EQ -> checkRightB l a (substituteR rl rr)
             GT -> checkRightB l a (deleteRr rl ra rr)
+            EQ -> checkRightB l a (substituteR rl rr)
       )
   deleteLl l a r =
     set'
@@ -600,20 +602,20 @@ delete' a0 =
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftL (deleteLl ll la lr) a r
-            EQ -> checkLeftL (substituteL ll lr) a r
             GT -> checkLeftL (deleteLr ll la lr) a r
+            EQ -> checkLeftL (substituteL ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> L (deleteBl ll la lr) a r
-            EQ -> checkLeftL' (substituteBr ll lr) a r
             GT -> L (deleteBr ll la lr) a r
+            EQ -> checkLeftL' (substituteBr ll lr) a r
       )
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> checkLeftL (deleteRl ll la lr) a r
-            EQ -> checkLeftL (substituteR ll lr) a r
             GT -> checkLeftL (deleteRr ll la lr) a r
+            EQ -> checkLeftL (substituteR ll lr) a r
       )
       l
   deleteLr l a =
@@ -622,20 +624,20 @@ delete' a0 =
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightL l a (deleteLl rl ra rr)
-            EQ -> checkRightL l a (substituteL rl rr)
             GT -> checkRightL l a (deleteLr rl ra rr)
+            EQ -> checkRightL l a (substituteL rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> L l a (deleteBl rl ra rr)
-            EQ -> checkRightL' l a (substituteBl rl rr)
             GT -> L l a (deleteBr rl ra rr)
+            EQ -> checkRightL' l a (substituteBl rl rr)
       )
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> checkRightL l a (deleteRl rl ra rr)
-            EQ -> checkRightL l a (substituteR rl rr)
             GT -> checkRightL l a (deleteRr rl ra rr)
+            EQ -> checkRightL l a (substituteR rl rr)
       )
   rebalanceR l a =
     set'
@@ -912,18 +914,18 @@ insert' a0 =
   insertR l a r =
     case compare a0 a of
       LT -> insertRl l a r
-      EQ -> R l a0 r
       GT -> insertRr l a r
+      EQ -> R l a0 r
   insertB l a r =
     case compare a0 a of
       LT -> insertBl l a r
-      EQ -> B l a0 r
       GT -> insertBr l a r
+      EQ -> B l a0 r
   insertL l a r =
     case compare a0 a of
       LT -> insertLl l a r
-      EQ -> L l a0 r
       GT -> insertLr l a r
+      EQ -> L l a0 r
   insertRl l a r =
     set'
       (B (B E a0 E) a r)
@@ -989,8 +991,8 @@ insert' a0 =
       ( \rl ra rr _ _ ->
           case compare a0 ra of
             LT -> insertRrl l a rl ra rr
-            EQ -> R l a (B rl a0 rr)
             GT -> insertRrr l a rl ra rr
+            EQ -> R l a (B rl a0 rr)
       )
       (\rl ra rr _ _ -> R l a (insertR rl ra rr))
   insertLl l a r =
@@ -1000,8 +1002,8 @@ insert' a0 =
       ( \ll la lr _ _ ->
           case compare a0 la of
             LT -> insertLll ll la lr a r
-            EQ -> L (B ll a0 lr) a r
             GT -> insertLlr ll la lr a r
+            EQ -> L (B ll a0 lr) a r
       )
       (\ll la lr _ _ -> L (insertR ll la lr) a r)
       l
